import { serial, pgTable, integer } from "drizzle-orm/pg-core"
import { userTable } from "../schema"
import { relations } from "drizzle-orm"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const followRelationTable = pgTable('followers', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
  follower_id: integer('follower_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
})

export const followRelationTableRelations = relations(
  followRelationTable,
  ({ one }) => ({
    follower: one(userTable, {
      fields: [followRelationTable.follower_id],
      references: [userTable.id]
    }),
    followed: one(userTable, {
      fields: [followRelationTable.user_id],
      references: [userTable.id]
    }),
  })
)

const followRelationModelSchema = createSelectSchema(followRelationTable)
const followRelationDtoSchema = followRelationModelSchema
const followRelationInputSchema = createInsertSchema(followRelationTable)
const followRelationUpdateSchema = followRelationInputSchema
  .partial()
  .required({id: true})

export type FollowRelationModel = z.infer<typeof followRelationModelSchema>
export type FollowRelationDto = z.infer<typeof followRelationDtoSchema>
export type FollowRelationInput = z.infer<typeof followRelationInputSchema>
export type FollowRelationUpdate = z.infer<typeof followRelationUpdateSchema>

export const followRelationSchemas = {
  model: followRelationModelSchema,
  dto: followRelationDtoSchema,
  input: followRelationInputSchema,
  update: followRelationUpdateSchema
}

export default followRelationTable