import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import { resourceTable, userTable } from "../schema";
import { relations } from "drizzle-orm";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";

const resourceLikesTable = pgTable("resource_likes", {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer("user_id")
    .notNull()
    .references(() => userTable.id, { onDelete: 'cascade' }),
  resource_id: integer("resource_id")
    .notNull()
    .references(() => resourceTable.id, { onDelete: 'cascade' })
})

export const resourceLikesRelations = relations(resourceLikesTable, ({ one }) => ({
  user: one(userTable, {
    fields: [resourceLikesTable.user_id],
    references: [userTable.id]
  }),
  resource: one(resourceTable, {
    fields: [resourceLikesTable.resource_id],
    references: [resourceTable.id]
  })
}))

const resourceLikesModelSchema = createSelectSchema(resourceLikesTable)
const resourceLikesDtoSchema = resourceLikesModelSchema
const resourceLikesInputSchema = createInsertSchema(resourceLikesTable)
const resourceLikesUpdateSchema = resourceLikesInputSchema.partial().required({ user_id: true, resource_id: true })

export type ResourceLikesModel = z.infer<typeof resourceLikesModelSchema>
export type ResourceLikesDto = z.infer<typeof resourceLikesDtoSchema>
export type ResourceLikesInput = z.infer<typeof resourceLikesInputSchema>
export type ResourceLikesUpdate = z.infer<typeof resourceLikesUpdateSchema>

export const resourceLikesSchemas = {
  resourceLikesModelSchema,
  resourceLikesDtoSchema,
  resourceLikesInputSchema,
  resourceLikesUpdateSchema,
}

export default resourceLikesTable
