import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import resourceTable from "../schema/resource.schema";
import subjectsTable from "../schema/subject.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import { relations } from "drizzle-orm";

const resourceSubjectsTable = pgTable('resource_subjects', {
    id: serial('id').primaryKey()
      .unique()
      .notNull(),
    resource_id: integer('resource_id').notNull().references(() => resourceTable.id, {onDelete:'cascade'}),
    subject_id: integer('subject_id').notNull().references(() => subjectsTable.id, {onDelete:'cascade'})
})

export const resourceSubjectsRelations = relations(resourceSubjectsTable, ({one}) => ({
    resource: one(resourceTable, {
        fields: [resourceSubjectsTable.resource_id],
        references: [resourceTable.id]
    }),
    subject: one(subjectsTable, {
        fields: [resourceSubjectsTable.subject_id],
        references: [subjectsTable.id]
    })
}))

const resourceSubjectsModelSchema = createSelectSchema(resourceSubjectsTable)
const resourceSubjectsDtoSchema = resourceSubjectsModelSchema.omit({})
const resourceSubjectsInputSchema = createInsertSchema(resourceSubjectsTable)
const resourceSubjectsUpdateSchema = resourceSubjectsInputSchema

export type ResourceSubjectsModel = z.infer<typeof resourceSubjectsModelSchema>
export type ResourceSubjectsDto = z.infer<typeof resourceSubjectsDtoSchema>
export type ResourceSubjectsInput = z.infer<typeof resourceSubjectsInputSchema>
export type ResourceSubjectsUpdate = z.infer<typeof resourceSubjectsUpdateSchema>



export const resourceSubjectsSchema = {
    model: resourceSubjectsModelSchema,
    dto: resourceSubjectsDtoSchema,
    input: resourceSubjectsInputSchema,
    update: resourceSubjectsUpdateSchema,
}

export default resourceSubjectsTable