import { serial, pgTable, integer } from "drizzle-orm/pg-core"
import { institutionTable, userTable } from "../schema"
import { relations } from "drizzle-orm"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const userInstitutionRelationTable = pgTable('user_institutions', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
  institution_id: integer('institution_id')
    .references(() => institutionTable.id, {onDelete: 'cascade'})
    .notNull(),
})

export const userInstitutionRelationTableRelations = relations(
  userInstitutionRelationTable,
  ({ one }) => ({
    user: one(userTable, {
      fields: [userInstitutionRelationTable.user_id],
      references: [userTable.id]
    }),
    institution: one(institutionTable, {
      fields: [userInstitutionRelationTable.institution_id],
      references: [institutionTable.id]
    }),
  })
)

const userInstitutionRelationModelSchema = createSelectSchema(userInstitutionRelationTable)
const userInstitutionRelationDtoSchema = userInstitutionRelationModelSchema
const userInstitutionRelationInputSchema = createInsertSchema(userInstitutionRelationTable)
const userInstitutionRelationUpdateSchema = userInstitutionRelationInputSchema
  .partial()
  .required({id: true})

export type UserInstitutionRelationModel = z.infer<typeof userInstitutionRelationModelSchema>
export type UserInstitutionRelationDto = z.infer<typeof userInstitutionRelationDtoSchema>
export type UserInstitutionRelationInput = z.infer<typeof userInstitutionRelationInputSchema>
export type UserInstitutionRelationUpdate = z.infer<typeof userInstitutionRelationUpdateSchema>

export const userInstitutionRelationSchemas = {
  model: userInstitutionRelationModelSchema,
  dto: userInstitutionRelationDtoSchema,
  input: userInstitutionRelationInputSchema,
  update: userInstitutionRelationUpdateSchema
}

export default userInstitutionRelationTable