import { serial, pgTable, integer } from "drizzle-orm/pg-core"
import { itemsTable, userTable } from "../schema"
import { relations } from "drizzle-orm"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const userItemRelationTable = pgTable('user_items', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
  item_id: integer('item_id')
    .references(() => itemsTable.id, {onDelete: 'cascade'})
    .notNull(),
})

export const userItemRelationTableRelations = relations(
  userItemRelationTable,
  ({ one }) => ({
    user: one(userTable, {
      fields: [userItemRelationTable.user_id],
      references: [userTable.id]
    }),
    item: one(itemsTable, {
      fields: [userItemRelationTable.item_id],
      references: [itemsTable.id]
    }),
  })
)

const userItemRelationModelSchema = createSelectSchema(userItemRelationTable)
const userItemRelationDtoSchema = userItemRelationModelSchema
const userItemRelationInputSchema = createInsertSchema(userItemRelationTable)
const userItemRelationUpdateSchema = userItemRelationInputSchema
  .partial()
  .required({id: true})

export type UserItemRelationModel = z.infer<typeof userItemRelationModelSchema>
export type UserItemRelationDto = z.infer<typeof userItemRelationDtoSchema>
export type UserItemRelationInput = z.infer<typeof userItemRelationInputSchema>
export type UserItemRelationUpdate = z.infer<typeof userItemRelationUpdateSchema>

export const userItemRelationSchemas = {
  model: userItemRelationModelSchema,
  dto: userItemRelationDtoSchema,
  input: userItemRelationInputSchema,
  update: userItemRelationUpdateSchema
}

export default userItemRelationTable