import { Service } from "typedi";
import type { ActionInput, ActionModel, ActionUpdate } from "../schema/action.schema";
import db from "..";
import actionTable, { actionSchemas } from "../schema/action.schema";
import { eq } from "drizzle-orm";
import { z } from "zod";

@Service()
export class ActionRepo {
  async findMany():Promise<ActionModel[]>{
  
    return z.array(actionSchemas.model).parse(await db.query.actionTable.findMany())
  }
  async findById(id: ActionModel['id']): Promise<ActionModel | null>{
    const action = await db.query.actionTable.findFirst({
      where: eq(actionTable.id, id)
    })

    if(!action) return null
    return actionSchemas.model.parse(action)
  }
  async findByName(name: ActionModel['name']): Promise<ActionModel | null>{
    const action = await db.query.actionTable.findFirst({
      where: eq(actionTable.name, name)
    })

    if(!action) return null
    return actionSchemas.model.parse(action)
  }
  async create(action: ActionInput, tx?: db): Promise<ActionModel>{
    const repo = tx ?? db
    const [ret] = await repo
    .insert(actionTable)
    .values(action)
    .returning()

    return actionSchemas.model.parse(ret)
  }
  async update(action: ActionUpdate): Promise<ActionModel>{
    const [ret] = await db
    .update(actionTable)
    .set(action)
    .where(eq(actionTable.id, action.id))
    .returning()

    return actionSchemas.model.parse(ret)
  }
  async delete(id: ActionModel['id']): Promise<ActionModel>{
    const [ret] = await db
    .delete(actionTable)
    .where(eq(actionTable.id, id))
    .returning()

    return actionSchemas.model.parse(ret)
  }
}