import { Service } from "typedi";
import type {
  CollectionStatsInput,
  CollectionStatsModel,
} from '../schema/collection-stats.schema'
import db from "..";
import collectionStatsTable, { collectionStatsSchemas } from "../schema/collection-stats.schema";
import { eq, sql } from 'drizzle-orm'


@Service()
export class CollectionStatsRepo {
  async create(): Promise<CollectionStatsModel> {
    const collectionStats: CollectionStatsInput = {}
    const [ret] = await db
      .insert(collectionStatsTable)
      .values(collectionStats)
      .returning()

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret)
  }

  async updateViews(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ views: sql`${collectionStatsTable.views} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  }

  async updateDownloads(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ downloads: sql`${collectionStatsTable.downloads} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  }

  async updateLikes(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ likes: sql`${collectionStatsTable.likes} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  }

  async updateShares(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ shares: sql`${collectionStatsTable.shares} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  }

  async updateScore(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ score: sql`${collectionStatsTable.score} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  } 

  async updateFollows(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .update(collectionStatsTable)
      .set({ follows: sql`${collectionStatsTable.follows} + 1` }) 
      .where(eq(collectionStatsTable.id, id))
      .returning();

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret);
  }

  async delete(id: CollectionStatsModel['id']): Promise<CollectionStatsModel> {
    const [ret] = await db
      .delete(collectionStatsTable)
      .where(eq(collectionStatsTable.id, id))
      .returning()

    return collectionStatsSchemas.collectionStatsModelSchema.parse(ret)
  }

  async find(
    id: CollectionStatsModel['id']
  ): Promise<CollectionStatsModel | undefined> {
    const collectionStats = await db.query.collectionStatsTable.findFirst({
      where: eq(collectionStatsTable.id, id),
    })

    return collectionStatsSchemas.collectionStatsModelSchema.parse(collectionStats)
  }
}