import { Service } from "typedi";
import type { ComplaintInput, ComplaintModel, ComplaintUpdate } from "../schema/complaint.schema";
import db from "..";
import complaintTable, { complaintSchemas } from "../schema/complaint.schema";
import { eq } from "drizzle-orm";

@Service()
export class ComplaintRepo {
  async create(complaint: ComplaintInput): Promise<ComplaintModel> {
    const [ret] = await db
      .insert(complaintTable)
      .values(complaint)
      .returning()

    return complaintSchemas.complaintModelSchema.parse(ret)
  }

  async update(complaint: ComplaintUpdate): Promise<ComplaintModel> {
    const [ret] = await db
      .update(complaintTable)
      .set(complaint)
      .where(eq(complaintTable.id, complaint.id))
      .returning()

    return complaintSchemas.complaintModelSchema.parse(ret)
  }

  async evaluate(complaint: ComplaintUpdate): Promise<ComplaintModel> {
    complaint.evaluated_at = new Date()

    const [ret] = await db
      .update(complaintTable)
      .set(complaint)
      .where(eq(complaintTable.id, complaint.id))
      .returning()

    return complaintSchemas.complaintModelSchema.parse(ret)
  }

  async delete(id: ComplaintModel['id']): Promise<ComplaintModel> {
    const [ret] = await db
      .delete(complaintTable)
      .where(eq(complaintTable.id, id))
      .returning()

    return complaintSchemas.complaintModelSchema.parse(ret)
  }

  async find(id: ComplaintModel['id']): Promise<ComplaintModel | undefined> {
    const complaint = await db.query.complaintTable.findFirst({
      where: eq(complaintTable.id, id),
    })

    return complaint ? complaintSchemas.complaintModelSchema.parse(complaint) : undefined
  }

  async findMany(): Promise<ComplaintModel[]> {
    return complaintSchemas.complaintModelSchema
      .array()
      .parse(await db.query.complaintTable.findMany())
  }
}