import { Service } from "typedi";
import type { EducationalStagesInput, EducationalStagesModel, EducationalStagesUpdate } from "../schema/educational-stage.schema";
import db from "..";
import educationalStageTable, { educationalStageSchema } from "../schema/educational-stage.schema";
import { eq } from "drizzle-orm";


@Service()
export class educationalStageRepo {
    async create(educationalStage: EducationalStagesInput): Promise<EducationalStagesModel> {
        const [ret] = await db
            .insert(educationalStageTable)
            .values(educationalStage)
            .returning()

        return educationalStageSchema.model.parse(ret)
    }

    async update(educationalStage: EducationalStagesUpdate): Promise<EducationalStagesModel> {
        const [ret] = await db
            .update(educationalStageTable)
            .set(educationalStage)
            .where(eq(educationalStageTable.id, educationalStage.id))
            .returning()

        return educationalStageSchema.model.parse(ret)
    }

    async delete(id: EducationalStagesModel['id']): Promise<EducationalStagesModel> {
        const [ret] = await db
            .delete(educationalStageTable)
            .where(eq(educationalStageTable.id, id))
            .returning()

        return educationalStageSchema.model.parse(ret)
    }

    async find(id: EducationalStagesModel['id']): Promise<EducationalStagesModel | undefined> {
        const educationalStage = await db.query.educationalStageTable.findFirst({
            where: eq(educationalStageTable.id, id),
        })

        return educationalStageSchema.model.parse(educationalStage)
    }

    async findMany(): Promise<EducationalStagesModel[]> {
        const educationalStages = await db.query.educationalStageTable.findMany()
        // o codigo fica mais legivel com map, apesar disso estamos usando outra forma em stats-resources.repo.ts
        return educationalStages.map((educationalStage) => educationalStageSchema.model.parse(educationalStage))
    }

}