import { Service } from "typedi";
import followRelationTable, { followRelationSchemas, type FollowRelationInput, type FollowRelationModel, type FollowRelationUpdate } from "../relations/followers.relation";
import { z } from "zod";
import db from "@/db";
import { and, eq } from "drizzle-orm";

@Service()
export class FollowRepo{
  
  async findMany(): Promise<FollowRelationModel[]>{
    return z
      .array(followRelationSchemas.model)
      .parse(await db.query.followRelationTable.findMany())
  }

  async findByFollow(followRelation: FollowRelationInput): Promise<number> {
    const follow = await db.query.followRelationTable.findFirst({
      where: and(eq(followRelationTable.follower_id, followRelation.follower_id), eq(followRelationTable.user_id, followRelation.user_id))
    })

    if(!follow) return 0

    return follow.id
  }

  async findById(id: FollowRelationModel['id']): Promise<FollowRelationModel | null>{
    const followRelation = await db.query.followRelationTable.findFirst({
      where: eq(followRelationTable.id, id)
    })
    if(!followRelation) return null

    return followRelationSchemas.model.parse(followRelation)
  }

  async findByUserId(user_id: FollowRelationModel['user_id']): Promise<FollowRelationModel[]>{
    
    return z
      .array(followRelationSchemas.model).parse(await db.query.followRelationTable.findMany({
        where: eq(followRelationTable.user_id, user_id)}))
  }

  async findByFollowerId(follower_id: FollowRelationModel['follower_id']): Promise<FollowRelationModel[]>{
    return z
      .array(followRelationSchemas.model).parse(await db.query.followRelationTable.findMany({
        where: eq(followRelationTable.user_id, follower_id)}))
  }

  async create(followRelation: FollowRelationInput, tx?: db): Promise<FollowRelationModel> {
    const repo = tx ?? db
    console.log(followRelation)

    const [ret] = await repo
    .insert(followRelationTable)
    .values(followRelation)
    .returning()

    return followRelationSchemas.model.parse(ret)
  }

  async update(followRelation: FollowRelationUpdate): Promise<FollowRelationModel>{
    const [ret] = await db
      .update(followRelationTable)
      .set(followRelation)
      .where(eq(followRelationTable.id, followRelation.id))
      .returning()

      return followRelationSchemas.model.parse(ret)
  }

  async delete(id: FollowRelationModel['id']): Promise<FollowRelationModel>{
    const [ret] = await db
    .delete(followRelationTable)
    .where(eq(followRelationTable.id, id))
    .returning()

    return followRelationSchemas.model.parse(ret)
  }

}