import { Service } from "typedi";
import institutionTable, { institutionSchemas, type InstitutionInput, type InstitutionModel, type InstitutionUpdate } from "../schema/institution.schema";
import db from "@/db";
import { z } from "zod";
import { eq } from "drizzle-orm";

@Service()
export class InstitutionRepo{
  async findMany():Promise<InstitutionModel[]>{
  
    return z.array(institutionSchemas.model).parse(await db.query.institutionTable.findMany())
  }
  async findById(id: InstitutionModel['id']): Promise<InstitutionModel | null>{
    const role = await db.query.institutionTable.findFirst({
      where: eq(institutionTable.id, id)
    })

    if(!role) return null
    return institutionSchemas.model.parse(role)
  }
  async findByName(name: InstitutionModel['name']): Promise<InstitutionModel | null>{
    const role = await db.query.institutionTable.findFirst({
      where: eq(institutionTable.name, name)
    })

    if(!role) return null
    return institutionSchemas.model.parse(role)
  }
  async findByCity(city: InstitutionModel['city']): Promise<InstitutionModel | null>{
    const role = await db.query.institutionTable.findFirst({
      where: eq(institutionTable.city, city!)
    })

    if(!role) return null
    return institutionSchemas.model.parse(role)
  }
  async findByUf(uf: InstitutionModel['uf']): Promise<InstitutionModel | null>{
    const role = await db.query.institutionTable.findFirst({
      where: eq(institutionTable.uf, uf!)
    })

    if(!role) return null
    return institutionSchemas.model.parse(role)
  }
  async findByCep(cep: InstitutionModel['cep']): Promise<InstitutionModel | null>{
    const role = await db.query.institutionTable.findFirst({
      where: eq(institutionTable.cep, cep!)
    })

    if(!role) return null
    return institutionSchemas.model.parse(role)
  }
  async create(role: InstitutionInput, tx?: db): Promise<InstitutionModel>{
    const repo = tx ?? db
    const [ret] = await repo
    .insert(institutionTable)
    .values(role)
    .returning()

    return institutionSchemas.model.parse(ret)
  }
  async update(role: InstitutionUpdate): Promise<InstitutionModel>{
    const [ret] = await db
    .update(institutionTable)
    .set(role)
    .where(eq(institutionTable.id, role.id))
    .returning()

    return institutionSchemas.model.parse(ret)
  }
  async delete(id: InstitutionModel['id']): Promise<InstitutionModel>{
    const [ret] = await db
    .delete(institutionTable)
    .where(eq(institutionTable.id, id))
    .returning()

    return institutionSchemas.model.parse(ret)
  }
}