import { Service } from "typedi";
import type { ItemsInput, ItemsModel, ItemsUpdate } from "../schema/item.schema";
import db from "..";
import itemsTable, { itemsSchema } from "../schema/item.schema";
import { and, eq } from "drizzle-orm";


@Service()
export class ItemsRepo {
    async create(item: ItemsInput): Promise<ItemsModel> {
        const [ret] = await db
            .insert(itemsTable)
            .values(item)
            .returning()

        return itemsSchema.model.parse(ret)
    }

    async update(item: ItemsUpdate): Promise<ItemsModel> {
        const [ret] = await db
            .update(itemsTable)
            .set(item)
            .where(eq(itemsTable.id, item.id))
            .returning()

        return itemsSchema.model.parse(ret)
    }

    async delete(id: ItemsModel['id']): Promise<ItemsModel> {
        const [ret] = await db
            .delete(itemsTable)
            .where(eq(itemsTable.id, id))
            .returning()

        return itemsSchema.model.parse(ret)
    }

    async active(id: ItemsModel['id']): Promise<ItemsModel> {
        const [ret] = await db
            .update(itemsTable)
            .set({
                is_active: true
            })
            .where(eq(itemsTable.id, id))
            .returning()

        return itemsSchema.model.parse(ret)
    }


    async findById(id: ItemsModel['id']): Promise<ItemsModel | undefined> {
        const item = await db.query.itemsTable.findFirst({
            where: and(eq(itemsTable.id, id), eq(itemsTable.is_active, true))
        })

        return itemsSchema.model.parse(item)
    }

    async findMany(): Promise<ItemsModel[]> {
        return itemsSchema.model.array().parse(await db.query.itemsTable.findMany({where: eq(itemsTable.is_active, true)}))
    }
}