import { Service } from "typedi";
import db from "..";
import { eq } from "drizzle-orm";
import { z } from "zod";
import notificationRelationTable, { notificationSchemas, type NotificationInput, type NotificationModel, type NotificationUpdate } from "../relations/notification.relation";

@Service()
export class NotificationRepo {
  async findMany():Promise<NotificationModel[]>{
  
    return z.array(notificationSchemas.model).parse(await db.query.notificationRelationTable.findMany())
  }
  async findById(id: NotificationModel['id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.id, id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  async findByAction(action_id: NotificationModel['action_id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.action_id, action_id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  async findByActorUser(actor_user_id: NotificationModel['actor_user_id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.actor_user_id, actor_user_id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  async findByTargetUser(target_user_id: NotificationModel['target_user_id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.target_user_id, target_user_id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  async findByTargetResource(target_resource_id: NotificationModel['target_resource_id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.target_resource_id, target_resource_id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  async findByTargetCollection(target_collection_id: NotificationModel['target_collection_id']): Promise<NotificationModel | null>{
    const notification = await db.query.notificationRelationTable.findFirst({
      where: eq(notificationRelationTable.target_collection_id, target_collection_id)
    })

    if(!notification) return null
    return notificationSchemas.model.parse(notification)
  }
  
  async create(notification: NotificationInput, tx?: db): Promise<NotificationModel>{
    const repo = tx ?? db
    const [ret] = await repo
    .insert(notificationRelationTable)
    .values(notification)
    .returning()

    return notificationSchemas.model.parse(ret)
  }
  async update(notification: NotificationUpdate): Promise<NotificationModel>{
    const [ret] = await db
    .update(notificationRelationTable)
    .set(notification)
    .where(eq(notificationRelationTable.id, notification.id))
    .returning()

    return notificationSchemas.model.parse(ret)
  }
  async delete(id: NotificationModel['id']): Promise<NotificationModel>{
    const [ret] = await db
    .delete(notificationRelationTable)
    .where(eq(notificationRelationTable.id, id))
    .returning()

    return notificationSchemas.model.parse(ret)
  }
}