import { Service } from "typedi";
import db from "@/db";
import type { PasswordRecoveryInput, PasswordRecoveryModel, PasswordRecoveryUpdate } from "../schema/password-recovery.schema";
import passwordRecoveryTable, { passwordRecoverySchemas } from "../schema/password-recovery.schema";
import { eq } from "drizzle-orm";
import { z } from "zod";

@Service()
export class PasswordRecoveryRepo{
  async create(passTicket: PasswordRecoveryInput, tx?: db): Promise<PasswordRecoveryModel> {
    const repo = tx ?? db

    const [ret] = await repo
    .insert(passwordRecoveryTable)
    .values(passTicket)
    .returning()

    return passwordRecoverySchemas.model.parse(ret)
  }
  async update(resetTicket: PasswordRecoveryUpdate): Promise<PasswordRecoveryModel>{
    const [ret] = await db
      .update(passwordRecoveryTable)
      .set(resetTicket)
      .where(eq(passwordRecoveryTable.id, resetTicket.id))
      .returning()

      return passwordRecoverySchemas.model.parse(ret)
  }
  async findById(id: PasswordRecoveryModel['id']): Promise<PasswordRecoveryModel | null>{
    const passTicket = await db.query.passwordRecoveryTable.findFirst({
      where: eq(passwordRecoveryTable.id, id)
    })
    if (!passTicket) return null
    return passwordRecoverySchemas.model.parse(passTicket)
  }
  async findByUserId(userId: PasswordRecoveryModel['userId']): Promise<PasswordRecoveryModel[] | null>{
    const passTicket = await db.query.passwordRecoveryTable.findMany({
      where: eq(passwordRecoveryTable.userId, userId)
    })
    if (!passTicket) return null
    return z.array(passwordRecoverySchemas.model).parse(passTicket)
  }
  async delete(id: PasswordRecoveryModel['id']): Promise<PasswordRecoveryModel>{
    const [ret] = await db
    .delete(passwordRecoveryTable)
    .where(eq(passwordRecoveryTable.id, id))
    .returning()

    return passwordRecoverySchemas.model.parse(ret)
  }
}