import { Service } from "typedi";
import {type SubjectInput, type SubjectModel, type SubjectUpdate}  from "../schema/subject.schema";
import db from "..";
import subjectsTable, { subjectSchema } from "../schema/subject.schema";
import { eq } from "drizzle-orm";

@Service()
export class subjectsRepo {
    async create(subject: SubjectInput): Promise<SubjectModel> {
        const [ret] = await db
            .insert(subjectsTable)
            .values(subject)
            .returning()

        return subjectSchema.model.parse(ret)
    }

    async update(subject: SubjectUpdate): Promise<SubjectModel> {
        const [ret] = await db
            .update(subjectsTable)
            .set(subject)
            .where(eq(subjectsTable.id, subject.id))
            .returning()

        return subjectSchema.model.parse(ret)
    }

    async delete(id: SubjectModel['id']): Promise<SubjectModel> {
        const [ret] = await db
            .delete(subjectsTable)
            .where(eq(subjectsTable.id, id))
            .returning()

        return subjectSchema.model.parse(ret)
    }

    async find(id: SubjectModel['id']): Promise<SubjectModel | undefined> {
        const subject = await db.query.subjectsTable.findFirst({
            where: eq(subjectsTable.id, id),
        })

        return subjectSchema.model.parse(subject)
    }

    async findMany(): Promise<SubjectModel[]> {
        const subjects = await db.query.subjectsTable.findMany()
        // o codigo fica mais legivel com map, apesar disso estamos usando outra forma em stats-resources.repo.ts
        return subjects.map((subject) => subjectSchema.model.parse(subject))
    }
}