import { Service } from "typedi";
import { z } from "zod";
import db from "@/db";
import { and, eq } from "drizzle-orm";
import userInstitutionRelationTable, { userInstitutionRelationSchemas, type UserInstitutionRelationInput, type UserInstitutionRelationModel, type UserInstitutionRelationUpdate } from "../relations/user-institution.relation";

@Service()
export class UserInstitutionRepo{
  
  async findMany(): Promise<UserInstitutionRelationModel[]>{
    return z
      .array(userInstitutionRelationSchemas.model)
      .parse(await db.query.userInstitutionRelationTable.findMany())
  }

  async findByUserInstitution(userInstitutionRelation: UserInstitutionRelationInput): Promise<number> {
    const userInstitution = await db.query.userInstitutionRelationTable.findFirst({
      where: and(eq(userInstitutionRelationTable.institution_id, userInstitutionRelation.institution_id), eq(userInstitutionRelationTable.user_id, userInstitutionRelation.user_id))
    })

    if(!userInstitution) return 0

    return userInstitution.id
  }

  async findById(id: UserInstitutionRelationModel['id']): Promise<UserInstitutionRelationModel | null>{
    const userInstitutionRelation = await db.query.userInstitutionRelationTable.findFirst({
      where: eq(userInstitutionRelationTable.id, id)
    })
    if(!userInstitutionRelation) return null

    return userInstitutionRelationSchemas.model.parse(userInstitutionRelation)
  }

  async findByUserId(user_id: UserInstitutionRelationModel['user_id']): Promise<UserInstitutionRelationModel[]>{
    
    return z
      .array(userInstitutionRelationSchemas.model).parse(await db.query.userInstitutionRelationTable.findMany({
        where: eq(userInstitutionRelationTable.user_id, user_id)}))
  }

  async findByInstitutionId(userInstitution_id: UserInstitutionRelationModel['institution_id']): Promise<UserInstitutionRelationModel[]>{
    return z
      .array(userInstitutionRelationSchemas.model).parse(await db.query.userInstitutionRelationTable.findMany({
        where: eq(userInstitutionRelationTable.institution_id, userInstitution_id)}))
  }

  async create(userInstitutionRelation: UserInstitutionRelationInput, tx?: db): Promise<UserInstitutionRelationModel> {
    const repo = tx ?? db

    const [ret] = await repo
    .insert(userInstitutionRelationTable)
    .values(userInstitutionRelation)
    .returning()

    return userInstitutionRelationSchemas.model.parse(ret)
  }

  async update(userInstitutionRelation: UserInstitutionRelationUpdate): Promise<UserInstitutionRelationModel>{
    const [ret] = await db
      .update(userInstitutionRelationTable)
      .set(userInstitutionRelation)
      .where(eq(userInstitutionRelationTable.id, userInstitutionRelation.id))
      .returning()

      return userInstitutionRelationSchemas.model.parse(ret)
  }

  async delete(id: UserInstitutionRelationModel['id']): Promise<UserInstitutionRelationModel>{
    const [ret] = await db
    .delete(userInstitutionRelationTable)
    .where(eq(userInstitutionRelationTable.id, id))
    .returning()

    return userInstitutionRelationSchemas.model.parse(ret)
  }

}