import { Service } from "typedi";
import { z } from "zod";
import db from "@/db";
import { and, eq } from "drizzle-orm";
import userItemRelationTable, { userItemRelationSchemas, type UserItemRelationInput, type UserItemRelationModel, type UserItemRelationUpdate } from "../relations/user-item.relation";

@Service()
export class UserItemRepo{
  
  async findMany(): Promise<UserItemRelationModel[]>{
    return z
      .array(userItemRelationSchemas.model)
      .parse(await db.query.userItemRelationTable.findMany())
  }

  async findByUserItem(userItemRelation: UserItemRelationInput): Promise<number> {
    console.log(userItemRelation)
    const userItem = await db.query.userItemRelationTable.findFirst({
      where: and(eq(userItemRelationTable.item_id, userItemRelation.item_id), eq(userItemRelationTable.user_id, userItemRelation.user_id))
    })

    if(!userItem) return 0

    return userItem.id
  }

  async findById(id: UserItemRelationModel['id']): Promise<UserItemRelationModel | null>{
    const userItemRelation = await db.query.userItemRelationTable.findFirst({
      where: eq(userItemRelationTable.id, id)
    })
    if(!userItemRelation) return null

    return userItemRelationSchemas.model.parse(userItemRelation)
  }

  async findByUserId(user_id: UserItemRelationModel['user_id']): Promise<UserItemRelationModel[]>{
    
    return z
      .array(userItemRelationSchemas.model).parse(await db.query.userItemRelationTable.findMany({
        where: eq(userItemRelationTable.user_id, user_id)}))
  }

  async findByItemId(userItem_id: UserItemRelationModel['item_id']): Promise<UserItemRelationModel[]>{
    return z
      .array(userItemRelationSchemas.model).parse(await db.query.userItemRelationTable.findMany({
        where: eq(userItemRelationTable.item_id, userItem_id)}))
  }

  async create(userItemRelation: UserItemRelationInput, tx?: db): Promise<UserItemRelationModel> {
    const repo = tx ?? db
    console.log(userItemRelation)

    const [ret] = await repo
    .insert(userItemRelationTable)
    .values(userItemRelation)
    .returning()

    return userItemRelationSchemas.model.parse(ret)
  }

  async update(userItemRelation: UserItemRelationUpdate): Promise<UserItemRelationModel>{
    const [ret] = await db
      .update(userItemRelationTable)
      .set(userItemRelation)
      .where(eq(userItemRelationTable.id, userItemRelation.id))
      .returning()

      return userItemRelationSchemas.model.parse(ret)
  }

  async delete(id: UserItemRelationModel['id']): Promise<UserItemRelationModel>{
    const [ret] = await db
    .delete(userItemRelationTable)
    .where(eq(userItemRelationTable.id, id))
    .returning()

    return userItemRelationSchemas.model.parse(ret)
  }

}