import { Service } from "typedi";
import { z } from "zod";
import db from "@/db";
import { and, eq } from "drizzle-orm";
import userRoleRelationTable, { userRoleRelationSchemas, type UserRoleRelationInput, type UserRoleRelationModel, type UserRoleRelationUpdate } from "../relations/user-role.relation";

@Service()
export class UserRoleRepo{
  
  async findMany(): Promise<UserRoleRelationModel[]>{
    return z
      .array(userRoleRelationSchemas.model)
      .parse(await db.query.userRoleRelationTable.findMany())
  }

  async findByUserRole(userRoleRelation: UserRoleRelationInput): Promise<number> {
    console.log(userRoleRelation)
    const userRole = await db.query.userRoleRelationTable.findFirst({
      where: and(eq(userRoleRelationTable.role_id, userRoleRelation.role_id), eq(userRoleRelationTable.user_id, userRoleRelation.user_id))
    })

    if(!userRole) return 0

    return userRole.id
  }

  async findById(id: UserRoleRelationModel['id']): Promise<UserRoleRelationModel | null>{
    const userRoleRelation = await db.query.userRoleRelationTable.findFirst({
      where: eq(userRoleRelationTable.id, id)
    })
    if(!userRoleRelation) return null

    return userRoleRelationSchemas.model.parse(userRoleRelation)
  }

  async findByUserId(user_id: UserRoleRelationModel['user_id']): Promise<UserRoleRelationModel[]>{
    
    return z
      .array(userRoleRelationSchemas.model).parse(await db.query.userRoleRelationTable.findMany({
        where: eq(userRoleRelationTable.user_id, user_id)}))
  }

  async findByRoleId(userRole_id: UserRoleRelationModel['role_id']): Promise<UserRoleRelationModel[]>{
    return z
      .array(userRoleRelationSchemas.model).parse(await db.query.userRoleRelationTable.findMany({
        where: eq(userRoleRelationTable.role_id, userRole_id)}))
  }

  async create(userRoleRelation: UserRoleRelationInput, tx?: db): Promise<UserRoleRelationModel> {
    const repo = tx ?? db
    console.log(userRoleRelation)

    const [ret] = await repo
    .insert(userRoleRelationTable)
    .values(userRoleRelation)
    .returning()

    return userRoleRelationSchemas.model.parse(ret)
  }

  async update(userRoleRelation: UserRoleRelationUpdate): Promise<UserRoleRelationModel>{
    const [ret] = await db
      .update(userRoleRelationTable)
      .set(userRoleRelation)
      .where(eq(userRoleRelationTable.id, userRoleRelation.id))
      .returning()

      return userRoleRelationSchemas.model.parse(ret)
  }

  async delete(id: UserRoleRelationModel['id']): Promise<UserRoleRelationModel>{
    const [ret] = await db
    .delete(userRoleRelationTable)
    .where(eq(userRoleRelationTable.id, id))
    .returning()

    return userRoleRelationSchemas.model.parse(ret)
  }

}