import { sql } from "drizzle-orm";
import { pgTable, serial, timestamp, varchar } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";

const institutionTable = pgTable('institutions', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  name: varchar('name', {length: 255})
    .notNull(),
  uf: varchar('uf', {length: 2}),
  city: varchar('city', {length: 255}),
  cep: varchar('cep', {length: 10}),
  createdAt: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updatedAt: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),  
})

const institutionModelSchema = createSelectSchema(institutionTable)
const institutionDtoSchema = institutionModelSchema
const institutionInputSchema = createInsertSchema(institutionTable)
const institutionUpdateSchema = institutionInputSchema
  .partial()
  .required({ id: true })

export type InstitutionModel = z.infer<typeof institutionModelSchema>
export type InstitutionDto = z.infer<typeof institutionDtoSchema>
export type InstitutionInput = z.infer<typeof institutionInputSchema>
export type InstitutionUpdate = z.infer<typeof institutionUpdateSchema>

export const institutionSchemas = {
  model: institutionModelSchema,
  dto: institutionDtoSchema,
  input: institutionInputSchema,
  update: institutionUpdateSchema
}

export default institutionTable