import env from '@/env'
import { connection, db } from '@/db'
import { Table, getTableName, sql } from 'drizzle-orm'
import * as schema from '@/db/schema'
import * as seeds from '@/db/seeds'

if (!env.DB_SEEDING) {
  throw new Error('You must set DB_SEEDING to "true" when seeding')
}

async function resetTable(db: db, table: Table) {
  return db.execute(
    sql.raw(
      `TRUNCATE TABLE "${getTableName(table)}" RESTART IDENTITY CASCADE`
    )
  )
}


for (const table of schema.tables) {
  await resetTable(db, table)
}

await seeds.userStatsSeed(db)
await seeds.userSeed(db)
await seeds.objectTypeSeed(db)
await seeds.licenseSeed(db)
await seeds.resourceStatsSeed(db)
await seeds.resourceSeed(db)
await seeds.collectionStatsSeed(db)
await seeds.collectionSeed(db)
await seeds.subjectsData(db)
await seeds.submissionSeed(db)
await seeds.languageSeed(db)
await seeds.educationalStageSeed(db)
await seeds.roleSeed(db)
await seeds.complaintSeed(db)
await seeds.resourceLanguagesSeed(db)
await seeds.collectionLikesSeed(db)
await seeds.userRoleSeed(db)
await seeds.institutionSeed(db)
await seeds.collectionResourcesSeed(db)
await seeds.resourceSubjectsSeed(db)
await seeds.userInstitutionSeed(db)
await seeds.resourceLikesSeed(db)
await seeds.resourceEducationalStagesSeed(db)
await seeds.achievementSeed(db)
await seeds.itemsSeed(db)
await seeds.userAchievementsSeed(db)
await seeds.actionSeed(db)

await connection.end()
