import { UserStatsService } from "@/services/user-stats.service";
import { honoWithJwt } from "..";
import Container from "typedi";
import { zValidator } from "@hono/zod-validator";
import { userStatsSchemas } from "@/db/schema/user-stats.schema";
import { createApexError, HttpStatus } from "@/services/error.service";

const service = Container.get(UserStatsService)

export const userStatsRouter = honoWithJwt()
  /* Update é uma rota própria ou vai estar embutida nas rotas das devidas estatisticas */
  .post('/update',
    zValidator('json', userStatsSchemas.update),
    async (c) => {
      try{

        const input = await c.req.valid('json')
        const user_stats = userStatsSchemas.dto.parse(
          await service.update(input)
        )
              
        return c.json({user_stats})
      } catch (e) {
        return c.json(
          createApexError({
            status: 'error',
            message: 'could not update user stats',
            code: HttpStatus.BAD_REQUEST,
            path: c.req.routePath,
            suggestion: 'check the input and try again',
          }),
          HttpStatus.BAD_REQUEST
        )
      }
  })
  /* Nao deveria ter como deletar um user stats sozinho */
  .post('/delete/:id',
    zValidator('json', userStatsSchemas.update),
    async (c) =>{
      try{
        const id: number = +c.req.param('id')
        const user_stats = userStatsSchemas.dto.parse(
          await service.delete(id)
        )
        return c.json(user_stats)
      } catch (e){
        return c.json(
          createApexError({
            status: 'error',
            message: 'could delete user stats',
            code: HttpStatus.BAD_REQUEST,
            path: c.req.routePath,
            suggestion: 'check the input and try again',
          }),
          HttpStatus.BAD_REQUEST
        )
      }
  })