import type db from "@/db"
import type { NotificationInput, NotificationModel, NotificationUpdate } from "@/db/relations/notification.relation"
import { NotificationRepo } from "@/db/repo/notification.repo"
import { Inject, Service } from "typedi"

@Service()
export class NotificationService{
  @Inject()
  private readonly repo: NotificationRepo

  async findMany():Promise<NotificationModel[]>{
    
    return this.repo.findMany()
  }
  async findById(id: NotificationModel['id']): Promise<NotificationModel | null>{

    return this.repo.findById(id)
  }
  async findByAction(action_id: NotificationModel['action_id']): Promise<NotificationModel | null>{

    return this.repo.findByAction(action_id)
  }
  async findByActorUser(actor_user_id: NotificationModel['actor_user_id']): Promise<NotificationModel | null>{

    return this.repo.findByActorUser(actor_user_id)
  }
  async findByTargetUser(target_user_id: NotificationModel['target_user_id']): Promise<NotificationModel | null>{

    return this.repo.findByTargetUser(target_user_id)
  }
  async findByTargetResource(target_resource_id: NotificationModel['target_resource_id']): Promise<NotificationModel | null>{

    return this.repo.findByTargetResource(target_resource_id)
  }
  async findByTargetCollection(target_collection_id: NotificationModel['target_collection_id']): Promise<NotificationModel | null>{

    return this.repo.findByTargetCollection(target_collection_id)
  }
  async create(notification: NotificationInput, tx?: db): Promise<NotificationModel>{

    return this.repo.create(notification, tx)
  }
  async update(notification: NotificationUpdate): Promise<NotificationModel>{

    return this.repo.update(notification)
  }
  async delete(id: NotificationModel['id']): Promise<NotificationModel>{

    return this.repo.delete(id)
  }
}