/*/import  * as nodemailer from 'nodemailer';
import mailConfig from "./config";*/
import { Inject, Service } from "typedi";
import { PasswordRecoveryRepo } from '@/db/repo/password-recovery.repo';
import type { PasswordRecoveryModel } from '@/db/schema/password-recovery.schema';
import type db from '@/db';
import type { UserDto } from '@/db/schema/user.schema';

@Service()
export class PasswordRecoveryService {
    @Inject()
    private readonly repo: PasswordRecoveryRepo
    
    isNotValidTicket(resetTicket: PasswordRecoveryModel): boolean {
        
        if(resetTicket == null) return true
        const is_expired: boolean = Date.parse(resetTicket.expirationDate) <  Date.now()
        if(!resetTicket.validToken || is_expired || resetTicket.tokenUsed)
            return true
        return false
    }

    generateToken(userEmail: string, tokenExpirationDate: string): string{
        const hasher = new Bun.CryptoHasher("sha256")
        const hashKey = userEmail.concat(tokenExpirationDate)
        return hasher.update(hashKey).digest("base64")

    }
    
    generateExpirationDate(): string {
      const time_expire = new Date()
      time_expire.setHours(time_expire.getHours() + 1)
        
      return time_expire.toLocaleString('pt-br', { timeZone: 'America/Sao_Paulo'})
    }

    
    emailTemplatePasswordRecovery(resetTicket: PasswordRecoveryModel): string{
        const recover_link: string = process.env["URL"] + '/reset-password/'+resetTicket.tokenHash
        const token_link: string = recover_link + "?token=" + resetTicket.tokenHash
        
        const expirationDate = new Date(resetTicket.expirationDate)
        expirationDate.setHours(expirationDate.getHours() - 1)
        const requestedDate = expirationDate.toLocaleString('pt-br', { timeZone: 'America/Sao_Paulo'})
        const text = `
            <head>
                <style>

                    .container {
                    display: block;
                    text-align: center;
                        background-color: rgb(240, 240, 240);
                        box-shadow: 0 7px 29px 0 rgba(100, 100, 111, .2);
                        width: 50%;
                        margin: 0 auto;
                        padding: 60px 20px;
                        font-family: Arial, Helvetica, sans-serif;
                    }

                    button {
                        background-color: rgb(7, 29, 65);
                        color: white;
                        border: none;
                        border-radius: 5px;
                        padding: 10px;
                        font-size: 16px;
                        cursor: pointer;
                    }

                .container_msg {
                    text-align: center;	
                        padding: 0px 20px 0px 20px;
                        background-color: white;
                    }

                    button:hover {
                        background-color: rgb(7, 29, 65, 0.8);
                    }
                </style>
            </head>

            <body>
                <div class="container" >
                <a href="${process.env["URL"]}"></a>
                    <h1 style="color:rgb(7, 29, 65); font-size: 30px; font-weight: 800; margin-top: 10px;">Recuperação de senha</h1>
                    <p>Recebemos uma requisição de recuperação de senha em </p>
                <strong>${requestedDate}</strong>
                    <br>
                    <p> Para criar uma nova senha, acesse: </p>
                    <br>
                    <a href="${token_link}"><button>Recupere sua senha </button></a>
                </div>
            </body>

	    `
        // precisa pegar a logo do mecred
        // <a href="${process.env["URL"]}"><img src="${process.env["URL"]}/static/media/PNLD_logo.c06d68e4.png" width="133" height="53" /></a>
        return text
    }

    /*async sendEmail(receiver: string, subject: string, content: string): Promise<object>{
        const transporter = nodemailer.createTransporter(mailConfig);
        return await transporter.sendMail({
        from: "Não Responda <no-reply@c3sl.ufpr.br>",
        to: receiver,
        subject: subject,
        html: content
     })      
    }*/

    async create(user: UserDto, tx?: db): Promise<PasswordRecoveryModel>{
        const tokenExpirationDate = this.generateExpirationDate()
        const newTokenHash = this.generateToken(user?.email, tokenExpirationDate)
        const resetTicket = {
            userId: user!.id,
            tokenHash: newTokenHash,
            expirationDate: tokenExpirationDate,
            tokenUsed: false
        }
        return await this.repo.create(resetTicket, tx)
    }

    async findById(id: PasswordRecoveryModel['id']): Promise<PasswordRecoveryModel | null>{
        return await this.repo.findById(id)
    }

    async findByUserId(userId: PasswordRecoveryModel['userId']): Promise<PasswordRecoveryModel[] | null>{
        return await this.repo.findByUserId(userId)
    }

    async update(resetTicket: PasswordRecoveryModel): Promise<PasswordRecoveryModel>{
        return await this.repo.update(resetTicket)
    }

    async delete(id: PasswordRecoveryModel['id']): Promise<PasswordRecoveryModel>{
        return await this.repo.delete(id)
    }

}