import { resourceLanguages } from "@/db/repo/resource-language.repo";
import type { LanguageModel } from "@/db/schema/language.schema";
import type { ResourceModel } from "@/db/schema/resource.schema";
import { Inject, Service } from "typedi";

@Service()
export class ResourceLanguageService{
    @Inject()
    private readonly repo: resourceLanguages;

    // Associa um recurso a várias linguagens
    async associateResourceWithLanguages(resourceId: ResourceModel['id'], languages: LanguageModel['id'][]): Promise<void> {
        await this.repo.associateResourceWithLanguages(resourceId, languages);
    }

    // Obtém todas as linguagens associadas a um recurso
    async getLanguagesByResource(resourceId: ResourceModel['id']): Promise<LanguageModel[]> {
        return await this.repo.getLanguagesByResource(resourceId);
    }

    // Remove a associação entre um recurso e uma linguagem
    async removeLanguageFromResource(resourceId: ResourceModel['id'], languageId: LanguageModel['id'][]): Promise<void> {
        await this.repo.removeLanguageFromResource(resourceId, languageId);
    }

    // Verifica se a associação entre um recurso e uma linguagem existe
    async isAssociationExists(resourceId: ResourceModel['id'], languageId: LanguageModel['id']): Promise<boolean> {
        return await this.repo.isAssociationExists(resourceId, languageId);
    }

    // Obtém todos os recursos associados a uma linguagem
    async getResourcesByLanguage(languageId: LanguageModel['id']): Promise<ResourceModel[]> {
        return await this.repo.getResourcesByLanguage(languageId);
    }

    // Atualiza as associações de linguagens de um recurso
    async updateResourceLanguages(resourceId: ResourceModel['id'], newLanguageIds: LanguageModel['id'][]): Promise<void> {
        await this.repo.updateResourceLanguages(resourceId, newLanguageIds);
    }
}