import type db from "@/db";
import { 
  type UserInstitutionRelationInput,
  type UserInstitutionRelationModel, 
  type UserInstitutionRelationUpdate 
  } from "@/db/relations/user-institution.relation";
import { UserInstitutionRepo } from "@/db/repo/user-institution.repo";
import { Inject, Service } from "typedi";

@Service()
export class UserInstitutionRelationService {
  
  @Inject()
  private readonly repo: UserInstitutionRepo

  async findMany(): Promise<UserInstitutionRelationModel[]> {

    return await this.repo.findMany()
  }

  async findByUserInstitution(userInstitutionRelation: UserInstitutionRelationInput): Promise<number> {
    
    return await this.repo.findByUserInstitution(userInstitutionRelation)
  }

  async findById(
    id:  UserInstitutionRelationModel['id']
  ): Promise<UserInstitutionRelationModel | null>{
    
    return await this.repo.findById(id)
  }

  async findByUserId(
    userId:  UserInstitutionRelationModel['user_id']
  ): Promise<UserInstitutionRelationModel[]>{
    
    return await this.repo.findByUserId(userId)
  }

  async findByInstitutionId(
    institutionId:  UserInstitutionRelationModel['institution_id']
  ): Promise<UserInstitutionRelationModel[]>{
    
    return await this.repo.findByInstitutionId(institutionId)
  }

  async create(followRelation: UserInstitutionRelationInput, tx?: db): Promise<UserInstitutionRelationModel> {

    return this.repo.create(followRelation, tx)
  }

  async update(followRelation: UserInstitutionRelationUpdate): Promise<UserInstitutionRelationModel> {
    
    return this.repo.update(followRelation)
  }


  async delete(user: UserInstitutionRelationModel['id']): Promise<UserInstitutionRelationModel> {
    
    return this.repo.delete(user)
  }

}