import type db from "@/db";
import { 
  type UserItemRelationInput,
  type UserItemRelationModel, 
  type UserItemRelationUpdate 
  } from "@/db/relations/user-item.relation";
import { UserItemRepo } from "@/db/repo/user-item.repo";
import { Inject, Service } from "typedi";

@Service()
export class UserItemRelationService {
  
  @Inject()
  private readonly repo: UserItemRepo

  async findMany(): Promise<UserItemRelationModel[]> {

    return await this.repo.findMany()
  }

  async findByUserItem(userItemRelation: UserItemRelationInput): Promise<number> {
    
    return await this.repo.findByUserItem(userItemRelation)
  }

  async findById(
    id:  UserItemRelationModel['id']
  ): Promise<UserItemRelationModel | null>{
    
    return await this.repo.findById(id)
  }

  async findByUserId(
    userId:  UserItemRelationModel['user_id']
  ): Promise<UserItemRelationModel[]>{
    
    return await this.repo.findByUserId(userId)
  }

  async findByItemId(
    itemId:  UserItemRelationModel['item_id']
  ): Promise<UserItemRelationModel[]>{
    
    return await this.repo.findByItemId(itemId)
  }

  async create(followRelation: UserItemRelationInput, tx?: db): Promise<UserItemRelationModel> {

    return this.repo.create(followRelation, tx)
  }

  async update(followRelation: UserItemRelationUpdate): Promise<UserItemRelationModel> {
    
    return this.repo.update(followRelation)
  }


  async delete(user: UserItemRelationModel['id']): Promise<UserItemRelationModel> {
    
    return this.repo.delete(user)
  }

}