import type db from '@/db'
import { hashPassword } from '@/db/repo/auth.repo'
import { UserRepo } from '@/db/repo/user.repo'
import {
  userSchemas,
  type UserInput,
  type UserModel,
  type UserUpdate,
} from '@/db/schema/user.schema'
import { Inject, Service } from 'typedi'
import { z } from 'zod'

@Service()
export class UserService {  
  @Inject()
  private readonly repo: UserRepo

  getUpdateTime(): string {
    const deactivated_at = new Date()
        deactivated_at.setHours(deactivated_at.getHours() + 1)

        return deactivated_at.toLocaleString('pt-br', {timeZone: 'America/Sao_Paulo'})
  }

  async findMany(): Promise<UserModel[]> {
    return await this.repo.findMany()
  }

  async findByUsername(
    username: UserModel['username']
  ): Promise<UserModel | null> {
    return await this.repo.findByUsername(username)
  }

  async findByEmail(
    userEmail:  UserModel['email']
  ): Promise<UserModel | null>{
    return await this.repo.findByEmail(userEmail)
  }

  async findById(
    userId:  UserModel['id']
  ): Promise<UserModel | null>{
    return await this.repo.findById(userId)
  }

  async create(user: UserInput, tx?: db): Promise<UserModel> {
    user.password = hashPassword(user.password)

    return this.repo.create(user, tx)
  }

  async createMany(
    users: UserInput[],
    tx?: db
  ): Promise<UserModel[]> {
    for (const user of users) {
      user.password = hashPassword(user.password)
    }

    return this.repo.createMany(
      z.array(userSchemas.userInputSchema).parse(users),
      tx
    )
  }

  async uploadUsers(users: UserInput[]): Promise<UserModel[]> {
    for (const user of users) {
      user.password = hashPassword(user.password)
    }

    return this.repo.uploadUsers(users)
  }

  async update(user: UserUpdate): Promise<UserModel> {
    if(user.password)
      user.password = hashPassword(user.password)
    return this.repo.update(user)
  }

  async delete(user: UserModel): Promise<UserModel> {
    
    return this.repo.update(user)
  }

  async systemDelete(user: UserModel['id']): Promise<UserModel> {
    
    return this.repo.delete(user)
  }
}
